import json
from os.path import dirname, abspath, join
from os import getenv
import sys
from configparser import ConfigParser

VERSION_JSON_PATH = "version.json"
VERSION_TOML = "pyproject.toml"
VERSION_KEY = "tool.poetry"
VERSION_DIR = dirname(abspath(__file__))


def pull_version():
    """Pulls version from pyproject.toml and moves to version JSON"""
    rel_dir = join(VERSION_DIR, "..", "..")
    config_filename = join(rel_dir, VERSION_TOML)

    config = ConfigParser()
    config.read(config_filename)
    ver_data = dict(config[VERSION_KEY])

    clean = lambda s: s.replace('"', "").replace('"', "")

    for k, v in ver_data.items():
        ver_data[k] = clean(v)

    target_path = join(VERSION_DIR, VERSION_JSON_PATH)
    try:
        with open(target_path, "r") as f:
            print(">> Previous Version:")
            print(f.read())
    except:
        pass

    with open(target_path, "w") as f:
        print("<< New Version (path={}):".format(target_path))
        print(json.dumps(ver_data, indent=2))
        json.dump(ver_data, f, indent=2)


def parse_version():
    """Parses version JSON"""
    with open(join(VERSION_DIR, VERSION_JSON_PATH), "r") as f:
        ver = json.load(f)
    return ver


if __name__ == "__main__":
    pull_version()


ver = parse_version()


def get_version():
    v = ver["version"]
    print(v)
    return v


def verify_ci():
    tag = getenv("CIRCLE_TAG")

    if tag != ver["version"]:
        info = "Git tag: {0} does not match the version of this app: {1}".format(
            tag, ver["version"]
        )
        sys.exit(info)


__version__ = ver["version"]
__title__ = ver["name"]
__author__ = ver["authors"]
__homepage__ = ver["homepage"]
__repo__ = ver["repository"]



"""

.. module:: pydent

Submodules
==========

.. autosummary::
    :toctree: _autosummary

    aqhttp
    aqsession
    base
    exceptions
    interfaces
    models
    relationships
    browser
    planner
    marshaller
    utils

"""

# without nest_asyncio, Jupyter notebooks will crash if an async method is used
import nest_asyncio

nest_asyncio.apply()

from .aqsession import AqSession
from .base import ModelBase, ModelRegistry
from .utils import pprint
from .browser import Browser
from pydent._version import __version__, __title__, __author__, __homepage__, __repo__



import logging
import math
import pprint


def new_logger(name, level=logging.ERROR):
    """Instantiate a new logger with the given name. If channel handler exists, do not create a new one."""
    logger = logging.getLogger(name)
    logger.setLevel(logging.DEBUG)

    # make stream handler
    if not logger.handlers:
        ch = logging.StreamHandler()
        ch.setLevel(level)

        formatter = logging.Formatter(
            "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
        )
        ch.setFormatter(formatter)
        logger.addHandler(ch)
    else:
        ch = logger.handlers[0]
    return logger, ch


def condense_long_lists(d, max_list_len=20):
    """
    Condense the long lists in a dictionary

    :param d: dictionary to condense
    :type d: dict
    :param max_len: max length of lists to display
    :type max_len: int
    :return:
    :rtype:
    """
    if isinstance(d, dict):
        return_dict = {}
        for k in d:
            return_dict[k] = condense_long_lists(dict(d).pop(k))
        return dict(return_dict)
    elif isinstance(d, list):
        if len(d) > max_list_len:
            g = max_list_len / 2
            return d[: math.floor(g)] + ["..."] + d[-math.ceil(g) :]
        else:
            return d[:]
    return str(d)


class Loggable:
    def init_logger(self, name):
        self._logger_name = name
        new_logger(name)

    @property
    def _logger(self):
        return logging.getLogger(self._logger_name)

    @property
    def _log_handlers(self):
        return self._logger.handlers

    def set_verbose(self, verbose):
        if verbose:
            for h in self._log_handlers:
                h.setLevel(logging.INFO)
        else:
            for h in self._log_handlers:
                h.setLevel(logging.ERROR)

    def _pprint_data(
        self, data, width=80, depth=10, max_list_len=20, compact=True, indent=1
    ):
        return pprint.pformat(
            condense_long_lists(data, max_list_len=max_list_len),
            indent=indent,
            width=width,
            depth=depth,
            compact=compact,
        )

    def _info(self, msg):
        self._logger.info(msg)

    def _error(self, msg):
        self._logger.error(msg)
